@@ -1,22 +1,20 @@
 /*
  * playlist-api.h
- * Copyright 2010-2011 John Lindgren
+ * Copyright 2010-2012 John Lindgren
  *
- * This file is part of Audacious.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
  *
- * Audacious is free software: you can redistribute it and/or modify it under
- * the terms of the GNU General Public License as published by the Free Software
- * Foundation, version 2 or version 3 of the License.
+ * 1. Redistributions of source code must retain the above copyright notice,
+ *    this list of conditions, and the following disclaimer.
  *
- * Audacious is distributed in the hope that it will be useful, but WITHOUT ANY
- * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
- * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+ * 2. Redistributions in binary form must reproduce the above copyright notice,
+ *    this list of conditions, and the following disclaimer in the documentation
+ *    provided with the distribution.
  *
- * You should have received a copy of the GNU General Public License along with
- * Audacious. If not, see <http://www.gnu.org/licenses/>.
- *
- * The Audacious team does not consider modular code linking to Audacious or
- * using our public API to be a derived work.
+ * This software is provided "as is" and without any warranty, express or
+ * implied. In no event shall the authors be liable for any damages arising from
+ * the use of this software.
  */
 
 /* Do not include this file directly; use playlist.h instead. */
@@ -31,8 +29,8 @@
  * least one playlist open.  The playlists are numbered starting from zero. */
 AUD_FUNC0 (int, playlist_count)
 
-/* Adds a new playlist before the one numbered <at>.  If <at> is negative or
- * equal to the number of playlists, adds a new playlist after the last one. */
+/* Adds a new playlist before the one numbered <at>.  If <at> is -1 or equal to
+ * the number of playlists, adds a new playlist after the last one. */
 AUD_VFUNC1 (playlist_insert, int, at)
 
 /* Moves a contiguous block of <count> playlists starting with the one numbered
@@ -42,9 +40,8 @@ AUD_VFUNC3 (playlist_reorder, int, from, int, to, int, count)
 /* Closes a playlist.  CAUTION: The playlist is not saved, and no confirmation
  * is presented to the user.  If <playlist> is the only playlist, a new playlist
  * is added.  If <playlist> is the active playlist, another playlist is marked
- * active.  If <playlist> is the one from which the last song played was taken,
- * playback is stopped.  In this case, calls to playlist_get_playing() will
- * return -1, and the behavior of drct_play() is unspecified. */
+ * active.  If <playlist> is the currently playing playlist, playback is
+ * stopped. */
 AUD_VFUNC1 (playlist_delete, int, playlist)
 
 /* Returns a unique non-negative integer which can be used to identify a given
@@ -70,22 +67,20 @@ AUD_VFUNC2 (playlist_set_title, int, playlist, const char *, title)
 /* Returns the title associated with a playlist. */
 AUD_FUNC1 (char *, playlist_get_title, int, playlist)
 
-/* Marks a playlist as active.  This is the playlist which the user will see and
- * on which most DRCT functions will take effect. */
+/* Sets the active playlist.  This is the playlist that user interfaces will
+ * show to the user. */
 AUD_VFUNC1 (playlist_set_active, int, playlist)
 
-/* Returns the number of the playlist marked active. */
+/* Returns the number of the active playlist. */
 AUD_FUNC0 (int, playlist_get_active)
 
-/* Sets the playlist in which playback will begin when drct_play() is called.
- * This does not mark the playlist as active.  If a song is already playing, it
- * will be stopped.  If <playlist> is negative, calls to playlist_get_playing()
- * will return -1, and the behavior of drct_play() is unspecified. */
+/* Sets the currently playing playlist.  Starts playback, resuming from the
+ * position last played if possible.  If <playlist> is -1 or if the requested
+ * playlist is empty, stops playback. */
 AUD_VFUNC1 (playlist_set_playing, int, playlist)
 
-/* Returns the number of the playlist from which the last song played was taken,
- * or -1 if that cannot be determined.  Note that this playlist may not be
- * marked active. */
+/* Returns the number of the currently playing playlist.  If no playlist is
+ * playing, returns -1. */
 AUD_FUNC0 (int, playlist_get_playing)
 
 /* Returns the number of a "blank" playlist.  The active playlist is returned if
@@ -136,9 +131,8 @@ AUD_VFUNC7 (playlist_entry_insert_filtered, int, playlist, int, at,
  void *, user, bool_t, play)
 
 /* Removes a contiguous block of <number> entries starting from the one numbered
- * <at> from a playlist.  If the last song played is in this block, playback is
- * stopped.  In this case, calls to playlist_get_position() will return -1, and
- * the behavior of drct_play() is unspecified. */
+ * <at> from a playlist.  If necessary, the playback position is moved elsewhere
+ * in the playlist and playback is restarted (or stopped). */
 AUD_VFUNC3 (playlist_entry_delete, int, playlist, int, at, int, number)
 
 /* Returns the filename of an entry. */
@@ -167,7 +161,9 @@ AUD_FUNC3 (char *, playlist_entry_get_title, int, playlist, int, entry,
 /* Returns three strings (title, artist, and album) describing an entry.  The
  * strings are pooled, and the usual cautions apply.  If <fast> is nonzero,
  * returns a "best guess" based on the entry's filename if metadata for the
- * entry has not yet been read.  May return NULL for any and all values. */
+ * entry has not yet been read.  The caller may pass NULL for any values that
+ * are not needed; NULL may also be returned for any values that are not
+ * available. */
 AUD_VFUNC6 (playlist_entry_describe, int, playlist, int, entry,
  char * *, title, char * *, artist, char * *, album, bool_t, fast)
 
@@ -176,15 +172,13 @@ AUD_VFUNC6 (playlist_entry_describe, int, playlist, int, entry,
 AUD_FUNC3 (int, playlist_entry_get_length, int, playlist, int, entry,
  bool_t, fast)
 
-/* Sets the entry which will be played (if this playlist is selected with
- * playlist_set_playing()) when drct_play() is called.  If a song from this
- * playlist is already playing, it will be stopped.  If <position> is negative,
- * calls to playlist_get_position() will return -1, and the behavior of
- * drct_play() is unspecified. */
+/* Moves the playback position to the beginning of the entry at <position>.  If
+ * <position> is -1, unsets the playback position.  If <playlist> is the
+ * currently playing playlist, playback is restarted (or stopped). */
 AUD_VFUNC2 (playlist_set_position, int, playlist, int, position)
 
-/* Returns the number of the entry which was last played from this playlist, or
- * -1 if that cannot be determined. */
+/* Returns the playback position, or -1 if it is not set.  Note that the
+ * position may be set even if <playlist> is not currently playing. */
 AUD_FUNC1 (int, playlist_get_position, int, playlist)
 
 /* Sets whether an entry is selected. */
@@ -206,10 +200,9 @@ AUD_VFUNC2 (playlist_select_all, int, playlist, bool_t, selected)
  * requested offset. */
 AUD_FUNC3 (int, playlist_shift, int, playlist, int, position, int, distance)
 
-/* Removes the selected entries from a playlist.  If the last song played is one
- * of these entries, playback is stopped.  In this case, calls to
- * playlist_get_position() will return -1, and the behavior of drct_play() is
- * unspecified. */
+/* Removes the selected entries from a playlist.  If necessary, the playback
+ * position is moved elsewhere in the playlist and playback is restarted (or
+ * stopped). */
 AUD_VFUNC1 (playlist_delete_selected, int, playlist)
 
 /* Sorts the entries in a playlist based on filename.  The callback function
@@ -311,11 +304,12 @@ AUD_FUNC0 (bool_t, playlist_update_pending)
  * have changed, returns zero. */
 AUD_FUNC3 (int, playlist_updated_range, int, playlist, int *, at, int *, count)
 
-/* Returns nonzero if entries are being added to a playlist in the background. */
+/* Returns nonzero if entries are being added to a playlist in the background.
+ * If <playlist> is -1, checks all playlists. */
 AUD_FUNC1 (bool_t, playlist_add_in_progress, int, playlist)
 
 /* Returns nonzero if entries in a playlist are being scanned for metadata in
- * the background. */
+ * the background.  If <playlist> is -1, checks all playlists. */
 AUD_FUNC1 (bool_t, playlist_scan_in_progress, int, playlist)
 
 /* --- PLAYLIST UTILITY API --- */
@@ -351,3 +345,17 @@ AUD_FUNC1 (bool_t, filename_is_playlist, const char *, filename)
 /* Saves the entries in a playlist to a playlist file.  The format of the file
  * is determined from the file extension.  Returns nonzero on success. */
 AUD_FUNC2 (bool_t, playlist_save, int, playlist, const char *, filename)
+
+/* added in Audacious 3.4 */
+
+/* Reverses the order of the selected entries in a playlist. */
+AUD_VFUNC1 (playlist_reverse_selected, int, playlist)
+
+/* Reorders the selected entries in a playlist randomly. */
+AUD_VFUNC1 (playlist_randomize_selected, int, playlist)
+
+/* Sets the entry which has keyboard focus (-1 means no entry). */
+AUD_VFUNC2 (playlist_set_focus, int, playlist_num, int, entry_num)
+
+/* Gets the entry which has keyboard focus (-1 means no entry). */
+AUD_FUNC1 (int, playlist_get_focus, int, playlist_num)
